@@ -1,4 +1,4 @@
-# $Header: /raid/cvsroot/DBIx/DBIx-SearchBuilder/SearchBuilder/Record/Cachable.pm,v 1.3 2001/05/16 00:49:48 jesse Exp $
+# $Header: /raid/cvsroot/DBIx/DBIx-SearchBuilder/SearchBuilder/Record/Cachable.pm,v 1.5 2001/05/28 20:29:34 jesse Exp $
 # by Matt Knopp <mhat@netlag.com>
 
 package DBIx::SearchBuilder::Record::Cachable; 
@@ -26,12 +26,22 @@ sub new () {
      $this->{'_CacheConfig'}=__CachableDefaults::_CacheConfig();
   }
 
-  $this->{'_RECORD_CACHE'}=\%_RECORD_CACHE;
   return ($this);
 }
 
 
 
+# Function: _RecordCache
+# Type    : private instance
+# Args    : none
+# Lvalue  : hash: RecordCache
+# Desc    : Returns a reference to the record cache hash
+
+sub _RecordCache {
+    my $this = shift;
+    return(\%_RECORD_CACHE);
+}
+
 
 # Function: LoadByCols
 # Type    : (overloaded) public instance
@@ -44,13 +54,13 @@ sub LoadByCols {
   ## Generate the cache key
   my $cache_key=$this->_gen_cache_key(%attr);
 
-  if (exists $this->{'_RECORD_CACHE'}{$cache_key}) { 
-    $cache_time = $this->{'_RECORD_CACHE'}{$cache_key}{'time'};
+  if (exists $this->_RecordCache->{$cache_key}) { 
+    $cache_time = $this->_RecordCache->{$cache_key}{'time'};
 
     ## Decide if the cache object is too old
     if ((time() - $cache_time) <= $this->{'_CacheConfig'}{'cache_for_sec'}) {
-      $this->_fetch($cache_key); 
-      return (1, "Fetched from cache");
+	$this->_fetch($cache_key); 
+	return (1, "Fetched from cache");
     }
     else { 
       $this->_gc_expired();
@@ -85,7 +95,7 @@ sub _Set () {
   my ($this, %attr) = @_; 
   my $cache_key = $this->{'_CacheConfig'}{'cache_key'};
 
-  if (exists $this->{'_RECORD_CACHE'}{$cache_key}) {
+  if (exists $this->_RecordCache->{$cache_key}) {
     $this->_expire();
   }
  
@@ -105,8 +115,8 @@ sub _Set () {
 sub _gc_expired () { 
   my ($this) = @_; 
   
-  foreach $cache_key (keys %{$this->{'_RECORD_CACHE'}}) {
-    my $cache_time = $this->{'_RECORD_CACHE'}{$cache_key}{'time'};  
+  foreach $cache_key (keys %{$this->_RecordCache}) {
+    my $cache_time = $this->_RecordCache->{$cache_key}{'time'};  
     $this->_expire($cache_key) 
       if ((time() - $cache_time) > $this->{'_CacheConfig'}{'cache_for_sec'});
   }
@@ -123,7 +133,7 @@ sub _gc_expired () {
 
 sub _expire (\$) {
   my ($this, $cache_key) = @_; 
-  delete $this->{'_RECORD_CACHE'}{$cache_key};
+  delete $this->_RecordCache->{$cache_key};
   return (1);
 }
 
@@ -140,7 +150,7 @@ sub _fetch () {
   my ($this, $cache_key) = @_;
 
   $this->{'values'}  = 
-    $this->{'_RECORD_CACHE'}{$cache_key}{'obj'}{'values'};
+    $this->_RecordCache->{$cache_key}{'obj'}{'values'};
   return(1); 
 }
 
@@ -156,8 +166,8 @@ sub _fetch () {
 sub _store (\$) { 
   my ($this, $cache_key) = @_; 
   $this->{'_CacheConfig'}{'cache_key'} = $cache_key;
-  $this->{'_RECORD_CACHE'}{$cache_key}{'obj'}=$this;
-  $this->{'_RECORD_CACHE'}{$cache_key}{'time'}=time();
+  $this->_RecordCache->{$cache_key}{'obj'}=$this;
+  $this->_RecordCache->{$cache_key}{'time'}=time();
   
   return(1);
 }
